package com.landray.kmss.sys.attachment.util;

import javax.imageio.stream.ImageOutputStream;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.IIOImage;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;
import javax.imageio.ImageWriter;
import java.awt.image.ImageObserver;
import org.apache.commons.io.FilenameUtils;
import java.awt.image.BufferedImage;
import javax.imageio.ImageReadParam;
import java.util.Iterator;
import java.io.ByteArrayOutputStream;
import javax.imageio.stream.ImageInputStream;
import java.io.IOException;
import java.awt.image.RenderedImage;
import java.awt.Rectangle;
import javax.imageio.ImageReader;
import javax.imageio.ImageIO;
import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import org.slf4j.LoggerFactory;
import java.util.List;
import org.slf4j.Logger;

public class ImageCropUtil
{
    private static final Logger LOGGER;
    public static final String[] CROP_KEYS;
    public static final Integer[] B_SIZE;
    public static final Integer[] M_SIZE;
    public static final Integer[] S_SIZE;
    private static final List<String> ALPHA_IMAGE;
    
    static {
        LOGGER = LoggerFactory.getLogger(ImageCropUtil.class);
        CROP_KEYS = new String[] { "_b", "_m", "_s" };
        B_SIZE = new Integer[] { 120, 120 };
        M_SIZE = new Integer[] { 60, 60 };
        S_SIZE = new Integer[] { 30, 30 };
        (ALPHA_IMAGE = new ArrayList<String>()).add("png");
        ImageCropUtil.ALPHA_IMAGE.add("gif");
    }
    
    public static String bytesToHexString(final byte[] src) {
        final StringBuilder stringBuilder = new StringBuilder();
        if (src == null || src.length <= 0) {
            return null;
        }
        for (int i = 0; i < src.length; ++i) {
            final int v = src[i] & 0xFF;
            final String hv = Integer.toHexString(v);
            if (hv.length() < 2) {
                stringBuilder.append(0);
            }
            stringBuilder.append(hv);
        }
        return checkType(stringBuilder.toString().toUpperCase());
    }
    
    public static void cropImage(final InputStream input, final OutputStream out, final int startX, final int satrtY, final int width, final int height, String suffix) throws Exception {
        ImageInputStream iis = null;
        ByteArrayOutputStream readSuffix = null;
        InputStream readSuffixStream = null;
        InputStream src = null;
        try {
            readSuffix = cloneInputStream(input);
            readSuffixStream = new ByteArrayInputStream(readSuffix.toByteArray());
            final byte[] b = new byte[3];
            readSuffixStream.read(b, 0, b.length);
            suffix = bytesToHexString(b);
            final Iterator<ImageReader> iterator = ImageIO.getImageReadersByFormatName(suffix);
            src = new ByteArrayInputStream(readSuffix.toByteArray());
            final ImageReader reader = iterator.next();
            iis = ImageIO.createImageInputStream(src);
            reader.setInput(iis, true);
            final ImageReadParam param = reader.getDefaultReadParam();
            final Rectangle rect = new Rectangle(startX, satrtY, width, height);
            param.setSourceRegion(rect);
            final BufferedImage bi = reader.read(0, param);
            ImageIO.write(bi, suffix, out);
        }
        catch (final Exception e) {
            throw e;
        }
        finally {
            try {
                if (iis != null) {
                    iis.close();
                }
                if (readSuffixStream != null) {
                    readSuffixStream.close();
                }
                if (src != null) {
                    src.close();
                }
                if (readSuffix != null) {
                    readSuffix.close();
                }
                if (input != null) {
                    input.close();
                }
            }
            catch (final IOException ex) {}
        }
        try {
            if (iis != null) {
                iis.close();
            }
            if (readSuffixStream != null) {
                readSuffixStream.close();
            }
            if (src != null) {
                src.close();
            }
            if (readSuffix != null) {
                readSuffix.close();
            }
            if (input != null) {
                input.close();
            }
        }
        catch (final IOException ex2) {}
    }
    
    public static void compress(final InputStream src, final OutputStream out, final int w, final int h, final String fileName) throws IOException {
        final String suffix = FilenameUtils.getExtension(fileName);
        final BufferedImage bufImg = ImageIO.read(src);
        final double wr = w * 1.0 / bufImg.getWidth();
        final double hr = h * 1.0 / bufImg.getHeight();
        double ratio = 0.0;
        if (wr < hr) {
            ratio = h * 1.0 / bufImg.getHeight();
        }
        else {
            ratio = w * 1.0 / bufImg.getWidth();
        }
        final int newWidth = (int)(bufImg.getWidth() * ratio);
        final int newHeight = (int)(bufImg.getHeight() * ratio);
        BufferedImage bfImage = null;
        if (ImageCropUtil.ALPHA_IMAGE.contains(suffix)) {
            bfImage = new BufferedImage(newWidth, newHeight, 2);
            bfImage.getGraphics().drawImage(bufImg.getScaledInstance(newWidth, newHeight, 4), 0, 0, null);
            ImageIO.write(bfImage, suffix, out);
            return;
        }
        bfImage = new BufferedImage(newWidth, newHeight, 1);
        bfImage.getGraphics().drawImage(bufImg.getScaledInstance(newWidth, newHeight, 4), 0, 0, null);
        final ImageWriter imageWriter = ImageIO.getImageWritersBySuffix("jpeg").next();
        final ImageOutputStream ios = ImageIO.createImageOutputStream(out);
        imageWriter.setOutput(ios);
        final JPEGImageWriteParam jpegParams = (JPEGImageWriteParam)imageWriter.getDefaultWriteParam();
        jpegParams.setCompressionMode(2);
        jpegParams.setCompressionQuality(1.0f);
        final IIOMetadata imageMetaData = imageWriter.getDefaultImageMetadata(new ImageTypeSpecifier(bfImage), jpegParams);
        imageWriter.write(imageMetaData, new IIOImage(bfImage, null, null), jpegParams);
        ios.close();
        imageWriter.dispose();
    }
    
    private static ByteArrayOutputStream cloneInputStream(final InputStream input) {
        try {
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            final byte[] buffer = new byte[1024];
            int len;
            while ((len = input.read(buffer)) > -1) {
                baos.write(buffer, 0, len);
            }
            baos.flush();
            return baos;
        }
        catch (final IOException e) {
            e.printStackTrace();
            return null;
        }
    }
    
    public static String checkType(final String xxxx) {
        switch (xxxx) {
            case "474946": {
                return "gif";
            }
            case "89504E": {
                return "png";
            }
            case "FFD8FF": {
                return "jpg";
            }
            default:
                break;
        }
        return "0000";
    }
}
